/*
 The software in this package is distributed under the GNU General
 Public License version 2 (with a special exception described below).

 A copy of GNU General Public License (GPL) is included in this distribution,
 in the file COPYING.GPL.

 As a special exception, if other files instantiate templates or use macros
 or inline functions from this file, or you compile this file and link it
 with other works to produce a work based on this file, this file
 does not by itself cause the resulting work to be covered
 by the GNU General Public License.

 However the source code for this file must still be made available
 in accordance with section (3) of the GNU General Public License.

 This exception does not invalidate any other reasons why a work based
 on this file might be covered by the GNU General Public License.
 */

#include "config/option/hostpingprotocoloption.h"

#include <iostream>
#include <sstream>
#include <string>

#include <boost/foreach.hpp>

#include <logfunc.hpp>

#include "boost_assert_handler.h"

using namespace std;
using boost::program_options::value;
using boost::program_options::variables_map;
using I2n::Logger::GlobalLogger;

//-----------------------------------------------------------------------------
// HostPingProtocolOption
//-----------------------------------------------------------------------------

HostPingProtocolOption::HostPingProtocolOption() :
    HostConfigurationOption(
        "host.ping-protocol",
        value< vector<string> >(),
        "Defines which protocol will be used to ping the destination."
    )
{
}

HostPingProtocolOption::~HostPingProtocolOption()
{
}

bool HostPingProtocolOption::parse(
        const variables_map& vm,
        Configuration *configuration
)
{
    size_t hosts_count = 0;
    size_t hosts_protocols_count = 0;
    bool parsed_success = false;

    // [host] ping-protocol
    if ( 1 <= vm.count( get_command_string() ) )
    {
        HostList hosts_list = configuration->get_hosts();
        HostList::iterator hosts_list_iterator = hosts_list.begin();
        hosts_count = hosts_list.size();

        vector<string> hosts_protocols_list = vm[ get_command_string() ].as< vector<string> >();
        hosts_protocols_count = hosts_protocols_list.size();

        BOOST_ASSERT( hosts_count >= hosts_protocols_count );

        BOOST_FOREACH( string protocols_string, hosts_protocols_list )
        {
            BOOST_ASSERT( !protocols_string.empty() );

            HostItem host_item = *hosts_list_iterator;
            PingProtocolList host_protocols = parse_protocol_list( protocols_string );
            host_item->set_ping_protocol_list( host_protocols );
            ++hosts_list_iterator;

            GlobalLogger.info() << get_command_string() << "="
                    << protocols_string << endl;
        }

        parsed_success = true;
    }

    set_hosts_count( hosts_protocols_count );

    BOOST_ASSERT( hosts_count == hosts_protocols_count );

    return parsed_success;
}

PingProtocolList HostPingProtocolOption::parse_protocol_list(
        const std::string &protocol_list_string
) const
{
    BOOST_ASSERT( !protocol_list_string.empty() );

    PingProtocolList protocol_list;
    string protocol_string;
    istringstream iss( protocol_list_string );

    while ( getline( iss, protocol_string, ',' ) )
    {
        BOOST_ASSERT( !protocol_string.empty());

        PingProtocol protocol = get_ping_protocol_from_string( protocol_string );
        protocol_list.push_back( protocol );

        GlobalLogger.debug() << "- " << protocol_string;

        BOOST_ASSERT( ( PingProtocol_First <= protocol ) && ( protocol <= PingProtocol_Last ) );                       //lint !e568
    }

    BOOST_ASSERT( 0 < protocol_list.size() );

    return protocol_list;
}
